@@ -16,18 +16,17 @@
  * limitations under the License.
  */
 
+#include "Statistics.hh"
 #include "orc/OrcFile.hh"
 #include "wrap/gmock.h"
 #include "wrap/gtest-wrapper.h"
-#include "Statistics.hh"
 
 #include <cmath>
 
 namespace orc {
 
   TEST(ColumnStatistics, intColumnStatistics) {
-    std::unique_ptr<IntegerColumnStatisticsImpl> intStats(
-      new IntegerColumnStatisticsImpl());
+    auto intStats = std::make_unique<IntegerColumnStatisticsImpl>();
 
     // initial state
     EXPECT_EQ(0, intStats->getNumberOfValues());
@@ -70,8 +69,7 @@ namespace orc {
     EXPECT_EQ(51, intStats->getSum());
 
     // test merge
-    std::unique_ptr<IntegerColumnStatisticsImpl> other(
-      new IntegerColumnStatisticsImpl());
+    auto other = std::make_unique<IntegerColumnStatisticsImpl>();
 
     other->setHasNull(true);
     other->increase(100);
@@ -107,8 +105,7 @@ namespace orc {
     intStats->merge(*other);
     EXPECT_FALSE(intStats->hasSum());
 
-    std::unique_ptr<IntegerColumnStatisticsImpl> intStats2(
-      new IntegerColumnStatisticsImpl());
+    auto intStats2 = std::make_unique<IntegerColumnStatisticsImpl>();
 
     intStats2->update(1, 1);
     EXPECT_TRUE(intStats2->hasSum());
@@ -117,8 +114,7 @@ namespace orc {
   }
 
   TEST(ColumnStatistics, doubleColumnStatistics) {
-    std::unique_ptr<DoubleColumnStatisticsImpl> dblStats(
-      new DoubleColumnStatisticsImpl());
+    auto dblStats = std::make_unique<DoubleColumnStatisticsImpl>();
 
     // initial state
     EXPECT_EQ(0, dblStats->getNumberOfValues());
@@ -160,8 +156,7 @@ namespace orc {
     EXPECT_TRUE(std::abs(-95454.5343 - dblStats->getMinimum()) < 0.00001);
 
     // test merge
-    std::unique_ptr<DoubleColumnStatisticsImpl> other(
-      new DoubleColumnStatisticsImpl());
+    auto other = std::make_unique<DoubleColumnStatisticsImpl>();
 
     other->setHasNull(true);
     other->increase(987);
@@ -183,8 +178,7 @@ namespace orc {
   }
 
   TEST(ColumnStatistics, stringColumnStatistics) {
-    std::unique_ptr<StringColumnStatisticsImpl> strStats(
-      new StringColumnStatisticsImpl());
+    auto strStats = std::make_unique<StringColumnStatisticsImpl>();
 
     EXPECT_FALSE(strStats->hasMinimum());
     EXPECT_FALSE(strStats->hasMaximum());
@@ -222,8 +216,7 @@ namespace orc {
   }
 
   TEST(ColumnStatistics, boolColumnStatistics) {
-    std::unique_ptr<BooleanColumnStatisticsImpl> boolStats(
-      new BooleanColumnStatisticsImpl());
+    auto boolStats = std::make_unique<BooleanColumnStatisticsImpl>();
 
     // initial state
     EXPECT_EQ(0, boolStats->getNumberOfValues());
@@ -240,8 +233,7 @@ namespace orc {
     EXPECT_EQ(3, boolStats->getTrueCount());
 
     // test merge
-    std::unique_ptr<BooleanColumnStatisticsImpl> other(
-      new BooleanColumnStatisticsImpl());
+    auto other = std::make_unique<BooleanColumnStatisticsImpl>();
 
     other->setHasNull(true);
     other->increase(100);
@@ -256,8 +248,7 @@ namespace orc {
   }
 
   TEST(ColumnStatistics, timestampColumnStatistics) {
-    std::unique_ptr<TimestampColumnStatisticsImpl> tsStats(
-      new TimestampColumnStatisticsImpl());
+    auto tsStats = std::make_unique<TimestampColumnStatisticsImpl>();
 
     EXPECT_FALSE(tsStats->hasMaximum() || tsStats->hasMaximum());
 
@@ -275,8 +266,7 @@ namespace orc {
     EXPECT_EQ(999999, tsStats->getMaximumNanos());
 
     // test merge
-    std::unique_ptr<TimestampColumnStatisticsImpl> other(
-      new TimestampColumnStatisticsImpl());
+    auto other = std::make_unique<TimestampColumnStatisticsImpl>();
 
     other->setMaximum(160);
     other->setMinimum(90);
@@ -289,8 +279,7 @@ namespace orc {
   }
 
   TEST(ColumnStatistics, dateColumnStatistics) {
-    std::unique_ptr<DateColumnStatisticsImpl> tsStats(
-      new DateColumnStatisticsImpl());
+    auto tsStats = std::make_unique<DateColumnStatisticsImpl>();
 
     EXPECT_FALSE(tsStats->hasMaximum() || tsStats->hasMaximum());
 
@@ -304,8 +293,7 @@ namespace orc {
     EXPECT_EQ(100, tsStats->getMinimum());
 
     // test merge
-    std::unique_ptr<DateColumnStatisticsImpl> other(
-      new DateColumnStatisticsImpl());
+    auto other = std::make_unique<DateColumnStatisticsImpl>();
 
     other->setMaximum(160);
     other->setMinimum(90);
@@ -316,7 +304,7 @@ namespace orc {
   }
 
   TEST(ColumnStatistics, otherColumnStatistics) {
-    std::unique_ptr<ColumnStatisticsImpl> stats(new ColumnStatisticsImpl());
+    auto stats = std::make_unique<ColumnStatisticsImpl>();
 
     EXPECT_EQ(0, stats->getNumberOfValues());
     EXPECT_FALSE(stats->hasNull());
@@ -332,8 +320,7 @@ namespace orc {
   }
 
   TEST(ColumnStatistics, decimalColumnStatistics) {
-    std::unique_ptr<DecimalColumnStatisticsImpl> decStats(
-      new DecimalColumnStatisticsImpl());
+    auto decStats = std::make_unique<DecimalColumnStatisticsImpl>();
 
     // initial state
     EXPECT_EQ(0, decStats->getNumberOfValues());
@@ -399,8 +386,7 @@ namespace orc {
   }
 
   TEST(ColumnStatistics, timestampColumnStatisticsWithNanos) {
-    std::unique_ptr<TimestampColumnStatisticsImpl> tsStats(
-      new TimestampColumnStatisticsImpl());
+    auto tsStats = std::make_unique<TimestampColumnStatisticsImpl>();
 
     // normal operations
     for (int32_t i = 1; i <= 1024; ++i) {
@@ -423,8 +409,7 @@ namespace orc {
     EXPECT_EQ(999, tsStats->getMinimumNanos());
 
     // test merge with no change
-    std::unique_ptr<TimestampColumnStatisticsImpl> other1(
-      new TimestampColumnStatisticsImpl());
+    auto other1 = std::make_unique<TimestampColumnStatisticsImpl>();
     for (int32_t i = 1; i <= 1024; ++i) {
       other1->update(i * 100, i * 1000);
       other1->increase(1);
@@ -436,8 +421,7 @@ namespace orc {
     EXPECT_EQ(999, tsStats->getMinimumNanos());
 
     // test merge with min/max change only in nano
-    std::unique_ptr<TimestampColumnStatisticsImpl> other2(
-      new TimestampColumnStatisticsImpl());
+    auto other2 = std::make_unique<TimestampColumnStatisticsImpl>();
     other2->update(102400, 1024002);
     other2->update(100, 998);
     tsStats->merge(*other2);
@@ -447,8 +431,7 @@ namespace orc {
     EXPECT_EQ(998, tsStats->getMinimumNanos());
 
     // test merge with min/max change in milli
-    std::unique_ptr<TimestampColumnStatisticsImpl> other3(
-      new TimestampColumnStatisticsImpl());
+    auto other3 = std::make_unique<TimestampColumnStatisticsImpl>();
     other3->update(102401, 1);
     other3->update(99, 1);
     tsStats->merge(*other3);
@@ -459,8 +442,7 @@ namespace orc {
   }
 
   TEST(ColumnStatistics, timestampColumnStatisticsProbubuf) {
-    std::unique_ptr<TimestampColumnStatisticsImpl> tsStats(
-      new TimestampColumnStatisticsImpl());
+    auto tsStats = std::make_unique<TimestampColumnStatisticsImpl>();
     tsStats->increase(2);
     tsStats->update(100);
     tsStats->update(200);
@@ -473,8 +455,7 @@ namespace orc {
     EXPECT_FALSE(pbStats.timestampstatistics().has_maximumnanos());
 
     StatContext ctx(true, nullptr);
-    std::unique_ptr<TimestampColumnStatisticsImpl> tsStatsFromPb(
-      new TimestampColumnStatisticsImpl(pbStats, ctx));
+    auto tsStatsFromPb = std::make_unique<TimestampColumnStatisticsImpl>(pbStats, ctx);
     EXPECT_EQ(100, tsStatsFromPb->getMinimum());
     EXPECT_EQ(200, tsStatsFromPb->getMaximum());
     EXPECT_EQ(0, tsStatsFromPb->getMinimumNanos());
@@ -498,4 +479,56 @@ namespace orc {
     EXPECT_EQ(9999, tsStatsFromPb->getMaximumNanos());
   }
 
-}
+  TEST(ColumnStatistics, collectionColumnStatistics) {
+    auto collectionStats = std::make_unique<CollectionColumnStatisticsImpl>();
+
+    // initial state
+    EXPECT_EQ(0, collectionStats->getNumberOfValues());
+    EXPECT_FALSE(collectionStats->hasNull());
+    EXPECT_FALSE(collectionStats->hasMinimumChildren());
+    EXPECT_FALSE(collectionStats->hasMaximumChildren());
+    EXPECT_TRUE(collectionStats->hasTotalChildren());
+    EXPECT_EQ(0, collectionStats->getTotalChildren());
+
+    // normal operations
+    collectionStats->increase(1);
+    EXPECT_EQ(1, collectionStats->getNumberOfValues());
+
+    collectionStats->increase(0);
+    EXPECT_EQ(1, collectionStats->getNumberOfValues());
+
+    collectionStats->increase(9999999999999999l);
+    EXPECT_EQ(10000000000000000l, collectionStats->getNumberOfValues());
+
+    collectionStats->update(10);
+    EXPECT_EQ(10, collectionStats->getMaximumChildren());
+    EXPECT_EQ(10, collectionStats->getMinimumChildren());
+
+    collectionStats->update(20);
+    EXPECT_EQ(20, collectionStats->getMaximumChildren());
+    EXPECT_EQ(10, collectionStats->getMinimumChildren());
+
+    EXPECT_EQ(30, collectionStats->getTotalChildren());
+    // test merge
+    auto other = std::make_unique<CollectionColumnStatisticsImpl>();
+
+    other->update(40);
+    other->update(30);
+
+    collectionStats->merge(*other);
+    EXPECT_EQ(40, other->getMaximumChildren());
+    EXPECT_EQ(30, other->getMinimumChildren());
+    EXPECT_EQ(40, collectionStats->getMaximumChildren());
+    EXPECT_EQ(10, collectionStats->getMinimumChildren());
+    EXPECT_EQ(100, collectionStats->getTotalChildren());
+
+    // test overflow
+    other->update(std::numeric_limits<uint64_t>::max());
+    EXPECT_FALSE(other->hasTotalChildren());
+    // test merge overflow
+    other->setTotalChildren(std::numeric_limits<uint64_t>::max() - 50);
+    EXPECT_EQ(std::numeric_limits<uint64_t>::max() - 50, other->getTotalChildren());
+    collectionStats->merge(*other);
+    EXPECT_FALSE(collectionStats->hasTotalChildren());
+  }
+}  // namespace orc
