@@ -183,9 +183,15 @@ public class WriterImpl implements WriterInternal, MemoryManager.Callback {
     this.encodingStrategy = opts.getEncodingStrategy();
     this.compressionStrategy = opts.getCompressionStrategy();
 
-    this.rowIndexStride = opts.getRowIndexStride();
+    // ORC-1362: if isBuildIndex=false, then rowIndexStride will be set to 0.
+    if (opts.getRowIndexStride() >= 0 && opts.isBuildIndex()) {
+      this.rowIndexStride = opts.getRowIndexStride();
+    } else {
+      this.rowIndexStride = 0;
+    }
+
+    this.buildIndex = rowIndexStride > 0;
 
-    this.buildIndex = opts.isBuildIndex() && (rowIndexStride > 0);
     if (buildIndex && rowIndexStride < MIN_ROW_INDEX_STRIDE) {
       throw new IllegalArgumentException("Row stride must be at least " +
           MIN_ROW_INDEX_STRIDE);
@@ -230,7 +236,7 @@ public class WriterImpl implements WriterInternal, MemoryManager.Callback {
 
     treeWriter = TreeWriter.Factory.create(schema, null, new StreamFactory());
 
-    LOG.info("ORC writer created for path: {} with stripeSize: {} options: {}",
+    LOG.debug("ORC writer created for path: {} with stripeSize: {} options: {}",
         path, stripeSize, unencryptedOptions);
   }
 
