@@ -297,6 +297,7 @@ public class TestOrcOutputFormat {
     writer.close(Reporter.NULL);
     Path path = new Path(workDir, "value.orc");
     Reader file = OrcFile.createReader(path, OrcFile.readerOptions(conf));
+    assertEquals(OrcConf.ROW_BATCH_SIZE.getDefaultValue(), file.options().getRowBatchSize());
     assertEquals(3000, file.getNumberOfRows());
     assertEquals(TYPE_STRING, file.getSchema().toString());
   }
@@ -327,6 +328,7 @@ public class TestOrcOutputFormat {
     writer.close(Reporter.NULL);
     Path path = new Path(workDir, "key.orc");
     Reader file = OrcFile.createReader(path, OrcFile.readerOptions(conf));
+    assertEquals(128, file.options().getRowBatchSize());
     assertEquals(2000, file.getNumberOfRows());
     assertEquals(TYPE_STRING, file.getSchema().toString());
   }
